local name, private = ...

local list = private.objects.list
local queue = private.objects.queue
local dict = private.objects.dict

local _, _, _, TOC_VERSION = GetBuildInfo()

private.baseArtifacts = {
    -- NOTE: These are ITEM IDs, not artifact info.
    { -- 1 (Dwarf)
        common = {
            63113,
            64339,
            63112,
            64340,
            63409,
            64362,
            66054,
            64342,
            64344,
            64368,
            63414,
            64337,
            63408,
            64659,
            64487,
            64367,
            64366,
            64483,
            63411,
            64371,
            64485,
            63410,
            64484,
            64343,
            63111,
            64486,
            63110,
        },
        rare = {
            64373,
            64372,
            64489,
            64488,
        },
    },
    { -- 2 (Draenei)
        common = {
            64440,
            64453,
            64442,
            64455,
            64454,
            64458,
            64444,
            64443,
        },
        rare = {
            64456,
            64457,
        },
    },
    { -- 3 (Fossil)
        common = {
            64355,
            63121,
            63109,
            64349,
            64385,
            64473,
            64350,
            64468,
            66056,
            66057,
            63527,
            64387,
        },
        rare = {
            60955,
            60954,
        },
    },
    { -- 4 (Night Elf)
        common = {
            64647,
            64379,
            63407,
            63525,
            64381,
            64357,
            63528,
            64356,
            63129,
            63130,
            64354,
            66055,
            63131,
            64382,
            63526,
            64648,
            64378,
            64650,
        },
        rare = {
            64646,
            64361,
            64358,
            64383,
            64643,
            64645,
            64651,
        },
    },
    { -- 5 (Nerubian)
        common = {
            64479,
            64477,
            64476,
            64475,
            64478,
            64474,
            64480,
        },
        rare = {
            64481,
            64482,
        },
    },
    { -- 6 (Orc)
        common = {
            64436,
            64421,
            64418,
            64417,
            64419,
            64420,
            64438,
            64437,
            64389,
        },
        rare = {
            64644,
        },
    },
    { -- 7 (Tol["vir)
        common = {
            64657,
            64652,
            64653,
            64656,
            64658,
            64654,
            64655,
        },
        rare = {
            60847,
            64881,
            64904,
            64883,
            64885,
            64880,
        },
    },
    { -- 8 (Troll)
        common = {
            64348,
            64346,
            63524,
            64375,
            63523,
            63413,
            63120,
            66058,
            64347,
            63412,
            63118,
            64345,
            64374,
            63115,
        },
        rare = {
            64377,
        },
    },
    { -- 9 (Vrykul)
        common = {
            64464,
            64462,
            64459,
            64461,
            64467,
        },
        rare = {
            64460,
        },
    },
}

private.patchedArtifacts = {
    [40100] = {
        { -- 1 (Dwarf)
            common = {
            },
            rare = {
            },
        },
        { -- 2 (Draenei)
            common = {
            },
            rare = {
            },
        },
        { -- 3 (Fossil)
            common = {
            },
            rare = {
                69821,
                69776,
                69764,
            },
        },
        { -- 4 (Night Elf)
            common = {
            },
            rare = {
            },
        },
        { -- 5 (Nerubian)
            common = {
            },
            rare = {
            },
        },
        { -- 6 (Orc)
            common = {
            },
            rare = {
            },
        },
        { -- 7 (Tol'vir)
            common = {
            },
            rare = {
            },
        },
        { -- 8 (Troll)
            common = {
            },
            rare = {
                69824,
                69777,
            },
        },
        { -- 9 (Vrykul)
            common = {
            },
            rare = {
                69775,
            },
        },
    },
}

-- Use the list of base and patched artifacts to build a complete list of artifacts
private.artifacts = {}
private.artifacts.isProcessed = false
function private.artifacts:process()
    if not private.artifacts.isProcessed then
        for i = 1, #private.baseArtifacts do
            -- First, copy each race from the base list into the full table
            private.artifacts[i] = {}
            private.artifacts[i].common = list()
            for k, v in pairs(private.baseArtifacts[i].common) do
                private.artifacts[i].common:add(v)
                -- Cache the item data
                GetItemInfo(v)
            end
            private.artifacts[i].rare = list()
            for k, v in pairs(private.baseArtifacts[i].rare) do
                private.artifacts[i].rare:add(v)
                -- Cache the item data
                GetItemInfo(v)
            end
        end
        
        for k, v in pairs(private.patchedArtifacts) do
            -- Then, copy every artifact added in a patch that is currently active into this list
            if k <= TOC_VERSION then
                for i = 1, #v do
                    if i > #private.artifacts then
                        private.artifacts[i] = {}
                        private.artifacts[i].common = list()
                        private.artifacts[i].rare = list()
                    end
                    for kk, vv in pairs(v[i].common) do
                        private.artifacts[i].common:add(vv)
                        -- Cache the item data
                        GetItemInfo(vv)
                    end
                    for kk, vv in pairs(v[i].rare) do
                        private.artifacts[i].rare:add(vv)
                        -- Cache the item data
                        GetItemInfo(vv)
                    end
                end
            end
        end
        
        private.artifacts.isProcessed = true
    end
end

private.baseLinks = {
    -- ["Item name"] = "Project name"
    -- English
    ["Crawling Claw"] = "Mummified Monkey Paw",
    -- French
    ["Epingle  cheveux en argent et malachite"] = "pingle  cheveux en argent et malachite",
    ["Etrange presse-papier en argent"] = "trange presse-papier en argent",
    ["Couteau de chasse us"] = "Couteau de chasseur us",
    ["D de jeu en os"] = "D en os",
}

private.links = { byItem = dict(), byProject = dict() } -- To be populated on load
function private.links:getProject(itemName)
    return self.byItem[itemName] or itemName
end
function private.links:getItem(projectName)
    return self.byProject[projectName] or projectName
end

private.travel = {
    spells = {
        -- spell ID, cooldown (in seconds), reagent (0 if none)
        { 556, 900, 0 }, -- Astral Recall
        { 3561, 0, 17031 }, -- Teleport: Stormwind
        { 3562, 0, 17031 }, -- Teleport: Ironforge
        { 3565, 0, 17031 }, -- Teleport: Darnassus
        { 32271, 0, 17031 }, -- Teleport: Exodar
        { 49359, 0, 17031 }, -- Teleport: Theramore
        { 3567, 0, 17031 }, -- Teleport: Orgrimmar
        { 3563, 0, 17031 }, -- Teleport: Undercity
        { 3566, 0, 17031 }, -- Teleport: Thunder Bluff
        { 32272, 0, 17031 }, -- Teleport: Silvermoon
        { 49358, 0, 17031 }, -- Teleport: Stonard
        { 33690, 0, 17031 }, -- Teleport: Shattrath (Alliance)
        { 35715, 0, 17031 }, -- Teleport: Shattrath (Horde)
        { 53140, 0, 17031 }, -- Teleport: Dalaran
        { 88342, 0, 17031 }, -- Teleport: Tol Barad (Alliance)
        { 88344, 0, 17031 }, -- Teleport: Tol Barad (Horde)
        { 18960, 0, 0 }, -- Teleport: Moonglade
        { 50977, 60, 0 }, -- Death Gate
    },
    items = {
        -- item ID, spell ID, cooldown (in seconds), consumed (boolean)
        { 6948, 8690, 1800, false }, -- Hearthstone
        { 28585, 39937, 1800, false }, -- Ruby Slippers
        { 54452, 75136, 1800, false }, -- Ethereal Portal
        { 64488, 94719, 1800, false }, -- The Innkeeper's Daughter
        { 37118, 48129, 1200, true }, -- Scroll of Recall
        { 44314, 60320, 1200, true }, -- Scroll of Recall II
        { 44315, 60321, 1200, true }, -- Scroll of Recall III
        { 58487, 80256, 60, true }, -- Potion of Deepholm
        { 18986, 23453, 14400, false }, -- Ultrasafe Transporter - Gadgetzan
        { 30544, 36941, 14400, false }, -- Ultrasafe Transporter - Toshley's Station
        { 18984, 23442, 14400, false }, -- Dimensional Ripper - Everlook
        { 30542, 36890, 14400, false }, -- Dimensional Ripper - Area 52
        { 48933, 67833, 14400, false }, -- Wormhole Generator: Northrend
        { 50287, 71436, 86400, false }, -- Boots of the Bay
        { 63352, 89157, 28800, false }, -- Shroud of Cooperation (Stormwind)
        { 63353, 89158, 28800, false }, -- Shroud of Cooperation (Orgrimmar)
        { 63206, 89157, 14400, false }, -- Wrap of Unity (Stormwind)
        { 63207, 89158, 14400, false }, -- Wrap of Unity (Orgrimmar)
        { 65360, 89157, 7200, false }, -- Cloak of Coordination (Stormwind)
        { 65274, 89158, 7200, false }, -- Cloak of Coordination (Orgrimmar)
        { 52251, 73324, 3600, false }, -- Jaina's Locket
        { 40586, 54406, 1800, false }, -- Band of the Kirin Tor
        { 45688, 54406, 1800, false }, -- Inscribed Band of the Kirin Tor
        { 48954, 54406, 1800, false }, -- Etched Band of the Kirin Tor
        { 51560, 54406, 1800, false }, -- Runed Band of the Kirin Tor
        { 44935, 54406, 1800, false }, -- Ring of the Kirin Tor
        { 45690, 54406, 1800, false }, -- Inscribed Ring of the Kirin Tor
        { 48956, 54406, 1800, false }, -- Etched Ring of the Kirin Tor
        { 51559, 54406, 1800, false }, -- Runed Ring of the Kirin Tor
        { 44934, 54406, 1800, false }, -- Loop of the Kirin Tor
        { 45689, 54406, 1800, false }, -- Inscribed Loop of the Kirin Tor
        { 48955, 54406, 1800, false }, -- Etched Loop of the Kirin Tor
        { 51558, 54406, 1800, false }, -- Runed Loop of the Kirin Tor
        { 40585, 54406, 1800, false }, -- Signet of the Kirin Tor
        { 45691, 54406, 1800, false }, -- Inscribed Signed of the Kirin Tor
        { 48957, 54406, 1800, false }, -- Etched Signet of the Kirin Tor
        { 51557, 54406, 1800, false }, -- Runed Signet of the Kirin Tor
    },
}

private.onUseItems = {
    -- item ID, spell ID, cooldown (in seconds), consumed (boolean)
    -- Dwarf
    { 64373, 94690, 600, false }, -- Chalice of the Mountain Kings
    { 64488, 94719, 1800, false }, -- The Innkeeper's Daughter
    -- Draenei
    { 64457, 82674, 43200, false }, -- The Last Relic of Argus
    { 64456, 91752, 180, false }, -- Arrival of the Naaru
    -- Fossil
    { 69776, 98445, 1800, false }, -- Ancient Amber
    -- Night Elf
    { 64358, 95007, 600, false }, -- Highborne Soul Mirror
    { 64383, 91749, 60, false }, -- Kaldorei Wind Chimes
    { 64361, 91743, 900, false }, -- Druid and Priest Statue Set
    { 64646, 91760, 600, false }, -- Bones of Transformation
    { 64651, 91771, 600, false }, -- Wisp Amulet
    -- Nerubian
    { 64481, 91754, 600, false }, -- Blessing of the Old God
    { 64482, 91756, 60, false }, -- Puzzle Box of Yogg-Saron
    -- Orc
    -- Nothing for orc...
    -- Tol'vir
    { 64881, 92146, 600, false }, -- Pendant of the Scarab Storm
    -- Troll
    { 69777, 98447, 1800, false }, -- Haunted War Drum
    -- Vrykul
    { 69775, 98444, 0, false }, -- Vrykul Drinking Horn
    -- Learnable items (pets, mounts)
    { 64372, 90523, 0, true }, -- Clockwork Gnome
    { 69821, 98571, 0, true }, -- Pterrordax Hatchling
    { 60955, 84752, 0, true }, -- Fossilized Hatchling
    { 60954, 84751, 0, true }, -- Fossilized Raptor
    { 60847, 84263, 0, true }, -- Crawling Claw
    { 64883, 92155, 0, true }, -- Scepter of Azj'Aqir
    { 69824, 98587, 0, true }, -- Voodoo Figurine
}

-- Cache the travel spells/items
for i=1, #private.travel.items do
    GetItemIcon(private.travel.items[i][1])
    GetItemInfo(private.travel.items[i][1])
end
for i=1, #private.travel.spells do
    GetSpellInfo(private.travel.spells[i][1])
end

