local name, private = ...

local list = private.objects.list
local dict = private.objects.dict

private.objects.data = {}

------------------------------------------------------------
------------------------- Artifact -------------------------
------------------------------------------------------------

private.objects.data.artifact = private.objects.base()

--function private.objects.data.artifact:new()
--    local r = private.objects.data.artifact.__parent.new(self)
--    
--
--    
--    return r
--end

function private.objects.data.artifact:init(race, rarity, itemID)
    -- Base values
    self.name = ""
    self.projectName = ""
    self.rarity = "common"
    self.completed = 0
    self.race = 0
    self.itemID = 0
    self.itemLink = ""
    
    self.race = race
    self.rarity = rarity
    self.itemID = itemID
    
    -- Get the item's info based on the ID
    local name, link = GetItemInfo(itemID)
    self.name = name
    self.itemLink = link
    
    -- If there's an item-project link, get the project's name
    self.projectName = private.links:getProject(self.name)
end

function private.objects.data.artifact:serialise()
    -- Returns a table with the necessary artifact information
    local r = {}
    
    r.itemID = self.itemID
    r.rarity = self.rarity
    r.race = self.race
    r.completed = self.completed
    
    return r
end

function private.objects.data.artifact:deserialise(tbl)
    -- Loads a table with serialised information about this artifact
    self:init(tbl.race, tbl.itemID, tbl.rarity, tbl.completed)
end

function private.objects.data.artifact.__eq(t1, t2)
    -- Checks whether two artifact objects are the same artifact
    -- The item ID should be sufficient for this
    return t1.itemID == t2.itemID
end

------------------------------------------------------------
--------------------------- Race ---------------------------
------------------------------------------------------------

private.objects.data.race = private.objects.base()

--function private.objects.data.race:new()
--    local r = private.objects.data.race.__parent.new(self)
--    
--    return r
--end

function private.objects.data.race:init(raceID)
    -- Base values
    self.name = ""
    self.raceID = 0
    self.artifacts = { common = dict(), rare = dict() }
    self.completed = { common = 0, rare = 0, total = 0 }
    self.numArtifacts = { common = 0, rare = 0, total = 0 }
    self.totalCompleted = 0
    self.regularFragments = 0
    self.currentArtifact = {
        artifact = "",
        fragments = 50,
        sockets = 0,
        name = "",
    }
    self.keystoneID = 0
    self.numKeystones = 0
    self.keystoneFragments = 0
    self.totalFragments = 0
    
    -- Record the race ID itself
    self.raceID = raceID
    
    -- Get the basic info
    local name, _, keystoneID, numColl, numReq = GetArchaeologyRaceInfo(raceID)
    self.name = name
    self.keystoneID = keystoneID
    self.regularFragments = numColl
    self.currentArtifact.fragments = numReq
    
    self:load()
    
    -- Register an event to retrieve artifact information
    -- Do NOT request this information here! That will
    -- result in the information being requested once per race.
    --self:RegisterEvent("ARTIFACT_HISTORY_READY")
end

function private.objects.data.race:update()
    -- Get the number of available keystones
    self.numKeystones = GetItemCount(self.keystoneID)
    
    -- Select this race's artifact
    SetSelectedArtifact(self.raceID)
    
    -- Socket all available keystones
    while SocketItemToArtifact() do end
    
    -- Count the number of fragments available and required
    local reg, key, req = GetArtifactProgress()
    self.regularFragments = reg
    self.keystoneFragments = key
    self.currentArtifact.fragments = req
    self.totalFragments = reg + key
    
    -- Get information about the current artifact
    local name, _, _, _, _, keystones = GetSelectedArtifactInfo()
    local itemName = private.links:getItem(name)
    self.currentArtifact.artifact = self:getArtifact(itemName)
    self.currentArtifact.sockets = keystones
    self.currentArtifact.name = name
    
    -- Load all information about completed artifacts
    --local num = #private.artifacts[self.raceID]
    local num = GetNumArtifactsByRace(self.raceID)
    self.completed.common = 0
    self.completed.rare = 0
    self.totalCompleted = 0
    for i = 1, num do
        local name, _, rarity, _, _, _, _, _, completed = GetArtifactInfoByRace(self.raceID, i)
        local itemName = private.links:getItem(name)
        local artifact = self:getArtifact(itemName)
        artifact.completed = completed
        self.totalCompleted = self.totalCompleted + completed
        if rarity == 0 then self.completed.common = self.completed.common + 1 end
        if rarity == 1 then self.completed.rare = self.completed.rare + 1 end
    end
    self.completed.total = self.completed.common + self.completed.rare
end

function private.objects.data.race:load()
    -- Load all information about all artifacts
    for i=1, #private.artifacts[self.raceID].common do
        local itemID = private.artifacts[self.raceID].common[i]
        local artifact = private.objects.data.artifact:new()
        artifact:init(self.raceID, "common", itemID)
        if not self.artifacts.common:get(artifact.name) then self.artifacts.common:set(artifact.name, artifact) end -- Don't add the artifact if it's already there!
        self.numArtifacts.common = self.numArtifacts.common + 1
    end
    for i=1, #private.artifacts[self.raceID].rare do
        local itemID = private.artifacts[self.raceID].rare[i]
        local artifact = private.objects.data.artifact:new()
        artifact:init(self.raceID, "rare", itemID)
        if not self.artifacts.rare:get(artifact.name) then self.artifacts.rare:set(artifact.name, artifact) end -- Don't add the artifact if it's already there!
        self.numArtifacts.rare = self.numArtifacts.rare + 1
    end
    self.numArtifacts.total = self.numArtifacts.common + self.numArtifacts.rare
end

--function private.objects.data.race:event(event, addonName)
--    --private.artifacts:process() -- TODO: The artifact list MUST be processed on load!
--    if event == "ARTIFACT_HISTORY_READY" then
--        self:load()
--    end
--    
--    self:update()
--end

function private.objects.data.race:getArtifact(itemName)
    return self.artifacts.common:get(itemName) or self.artifacts.rare:get(itemName)
end

function private.objects.data.race:Solve()
    -- Select the artifact,
    SetSelectedArtifact(self.raceID)
    -- then solve it
    SolveArtifact()
end

--RequestArtifactCompletionHistory() -- Only works on reload for now - TESTING ONLY
--private.artifacts:process()
--test = private.objects.data.race()
--test:init(1)